@@ -6,13 +6,13 @@ import dataclasses
 import sys
 from functools import partialmethod
 from types import FunctionType
-from typing import TYPE_CHECKING, Any, Callable, TypeVar, Union, cast, overload
+from typing import TYPE_CHECKING, Annotated, Any, Callable, Literal, TypeVar, Union, cast, overload
 
 from pydantic_core import PydanticUndefined, core_schema
 from pydantic_core import core_schema as _core_schema
-from typing_extensions import Annotated, Literal, Self, TypeAlias
+from typing_extensions import Self, TypeAlias
 
-from ._internal import _core_metadata, _decorators, _generics, _internal_dataclass
+from ._internal import _decorators, _generics, _internal_dataclass
 from .annotated_handlers import GetCoreSchemaHandler
 from .errors import PydanticUserError
 
@@ -26,7 +26,8 @@ _inspect_validator = _decorators.inspect_validator
 
 @dataclasses.dataclass(frozen=True, **_internal_dataclass.slots_true)
 class AfterValidator:
-    """Usage docs: https://docs.pydantic.dev/2.9/concepts/validators/#annotated-validators
+    """!!! abstract "Usage Documentation"
+        [field *after* validators](../concepts/validators.md#field-after-validator)
 
     A metadata class that indicates that a validation should be applied **after** the inner validation logic.
 
@@ -34,8 +35,8 @@ class AfterValidator:
         func: The validator function.
 
     Example:
-        ```py
-        from typing_extensions import Annotated
+        ```python
+        from typing import Annotated
 
         from pydantic import AfterValidator, BaseModel, ValidationError
 
@@ -86,7 +87,8 @@ class AfterValidator:
 
 @dataclasses.dataclass(frozen=True, **_internal_dataclass.slots_true)
 class BeforeValidator:
-    """Usage docs: https://docs.pydantic.dev/2.9/concepts/validators/#annotated-validators
+    """!!! abstract "Usage Documentation"
+        [field *before* validators](../concepts/validators.md#field-before-validator)
 
     A metadata class that indicates that a validation should be applied **before** the inner validation logic.
 
@@ -96,8 +98,8 @@ class BeforeValidator:
             JSON Schema (in validation mode).
 
     Example:
-        ```py
-        from typing_extensions import Annotated
+        ```python
+        from typing import Annotated
 
         from pydantic import BaseModel, BeforeValidator
 
@@ -127,7 +129,6 @@ class BeforeValidator:
             if self.json_schema_input_type is PydanticUndefined
             else handler.generate_schema(self.json_schema_input_type)
         )
-        metadata = _core_metadata.build_metadata_dict(js_input_core_schema=input_schema)
 
         info_arg = _inspect_validator(self.func, 'before')
         if info_arg:
@@ -136,11 +137,13 @@ class BeforeValidator:
                 func,
                 schema=schema,
                 field_name=handler.field_name,
-                metadata=metadata,
+                json_schema_input_schema=input_schema,
             )
         else:
             func = cast(core_schema.NoInfoValidatorFunction, self.func)
-            return core_schema.no_info_before_validator_function(func, schema=schema, metadata=metadata)
+            return core_schema.no_info_before_validator_function(
+                func, schema=schema, json_schema_input_schema=input_schema
+            )
 
     @classmethod
     def _from_decorator(cls, decorator: _decorators.Decorator[_decorators.FieldValidatorDecoratorInfo]) -> Self:
@@ -152,29 +155,46 @@ class BeforeValidator:
 
 @dataclasses.dataclass(frozen=True, **_internal_dataclass.slots_true)
 class PlainValidator:
-    """Usage docs: https://docs.pydantic.dev/2.9/concepts/validators/#annotated-validators
+    """!!! abstract "Usage Documentation"
+        [field *plain* validators](../concepts/validators.md#field-plain-validator)
 
     A metadata class that indicates that a validation should be applied **instead** of the inner validation logic.
 
+    !!! note
+        Before v2.9, `PlainValidator` wasn't always compatible with JSON Schema generation for `mode='validation'`.
+        You can now use the `json_schema_input_type` argument to specify the input type of the function
+        to be used in the JSON schema when `mode='validation'` (the default). See the example below for more details.
+
     Attributes:
         func: The validator function.
         json_schema_input_type: The input type of the function. This is only used to generate the appropriate
             JSON Schema (in validation mode). If not provided, will default to `Any`.
 
     Example:
-        ```py
-        from typing_extensions import Annotated
+        ```python
+        from typing import Annotated, Union
 
         from pydantic import BaseModel, PlainValidator
 
-        MyInt = Annotated[int, PlainValidator(lambda v: int(v) + 1)]
+        MyInt = Annotated[
+            int,
+            PlainValidator(
+                lambda v: int(v) + 1, json_schema_input_type=Union[str, int]  # (1)!
+            ),
+        ]
 
         class Model(BaseModel):
             a: MyInt
 
         print(Model(a='1').a)
         #> 2
+
+        print(Model(a=1).a)
+        #> 2
         ```
+
+        1. In this example, we've specified the `json_schema_input_type` as `Union[str, int]` which indicates to the JSON schema
+        generator that in validation mode, the input type for the `a` field can be either a `str` or an `int`.
     """
 
     func: core_schema.NoInfoValidatorFunction | core_schema.WithInfoValidatorFunction
@@ -204,7 +224,6 @@ class PlainValidator:
             serialization = None
 
         input_schema = handler.generate_schema(self.json_schema_input_type)
-        metadata = _core_metadata.build_metadata_dict(js_input_core_schema=input_schema)
 
         info_arg = _inspect_validator(self.func, 'plain')
         if info_arg:
@@ -213,14 +232,14 @@ class PlainValidator:
                 func,
                 field_name=handler.field_name,
                 serialization=serialization,  # pyright: ignore[reportArgumentType]
-                metadata=metadata,
+                json_schema_input_schema=input_schema,
             )
         else:
             func = cast(core_schema.NoInfoValidatorFunction, self.func)
             return core_schema.no_info_plain_validator_function(
                 func,
                 serialization=serialization,  # pyright: ignore[reportArgumentType]
-                metadata=metadata,
+                json_schema_input_schema=input_schema,
             )
 
     @classmethod
@@ -233,7 +252,8 @@ class PlainValidator:
 
 @dataclasses.dataclass(frozen=True, **_internal_dataclass.slots_true)
 class WrapValidator:
-    """Usage docs: https://docs.pydantic.dev/2.9/concepts/validators/#annotated-validators
+    """!!! abstract "Usage Documentation"
+        [field *wrap* validators](../concepts/validators.md#field-wrap-validator)
 
     A metadata class that indicates that a validation should be applied **around** the inner validation logic.
 
@@ -242,10 +262,9 @@ class WrapValidator:
         json_schema_input_type: The input type of the function. This is only used to generate the appropriate
             JSON Schema (in validation mode).
 
-    ```py
+    ```python
     from datetime import datetime
-
-    from typing_extensions import Annotated
+    from typing import Annotated
 
     from pydantic import BaseModel, ValidationError, WrapValidator
 
@@ -281,7 +300,6 @@ class WrapValidator:
             if self.json_schema_input_type is PydanticUndefined
             else handler.generate_schema(self.json_schema_input_type)
         )
-        metadata = _core_metadata.build_metadata_dict(js_input_core_schema=input_schema)
 
         info_arg = _inspect_validator(self.func, 'wrap')
         if info_arg:
@@ -290,14 +308,14 @@ class WrapValidator:
                 func,
                 schema=schema,
                 field_name=handler.field_name,
-                metadata=metadata,
+                json_schema_input_schema=input_schema,
             )
         else:
             func = cast(core_schema.NoInfoWrapValidatorFunction, self.func)
             return core_schema.no_info_wrap_validator_function(
                 func,
                 schema=schema,
-                metadata=metadata,
+                json_schema_input_schema=input_schema,
             )
 
     @classmethod
@@ -394,12 +412,13 @@ def field_validator(
     check_fields: bool | None = None,
     json_schema_input_type: Any = PydanticUndefined,
 ) -> Callable[[Any], Any]:
-    """Usage docs: https://docs.pydantic.dev/2.9/concepts/validators/#field-validators
+    """!!! abstract "Usage Documentation"
+        [field validators](../concepts/validators.md#field-validators)
 
     Decorate methods on the class indicating that they should be used to validate fields.
 
     Example usage:
-    ```py
+    ```python
     from typing import Any
 
     from pydantic import (
@@ -651,12 +670,13 @@ def model_validator(
     *,
     mode: Literal['wrap', 'before', 'after'],
 ) -> Any:
-    """Usage docs: https://docs.pydantic.dev/2.9/concepts/validators/#model-validators
+    """!!! abstract "Usage Documentation"
+        [Model Validators](../concepts/validators.md#model-validators)
 
     Decorate model methods for validation purposes.
 
     Example usage:
-    ```py
+    ```python
     from typing_extensions import Self
 
     from pydantic import BaseModel, ValidationError, model_validator
@@ -718,7 +738,7 @@ else:
         '''Generic type for annotating a type that is an instance of a given class.
 
         Example:
-            ```py
+            ```python
             from pydantic import BaseModel, InstanceOf
 
             class Foo:
@@ -797,7 +817,7 @@ else:
         @classmethod
         def __get_pydantic_core_schema__(cls, source: Any, handler: GetCoreSchemaHandler) -> core_schema.CoreSchema:
             original_schema = handler(source)
-            metadata = _core_metadata.build_metadata_dict(js_annotation_functions=[lambda _c, h: h(original_schema)])
+            metadata = {'pydantic_js_annotation_functions': [lambda _c, h: h(original_schema)]}
             return core_schema.any_schema(
                 metadata=metadata,
                 serialization=core_schema.wrap_serializer_function_ser_schema(
